package no.uia.dat101.sett10.grafikk;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JColorChooser;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.colorchooser.ColorSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;

import no.uia.dat101.oppg10.interfaces.TegneFigur;
import no.uia.dat101.sett10.grafikk.Model.Fasong;
import no.uia.dat101.sett10.grafikkobjekter.Linje;
import no.uia.dat101.sett10.grafikkobjekter.Oval;
import no.uia.dat101.sett10.grafikkobjekter.Rektangel;


public class View extends JFrame {

	// lag nytt tegnepanel (tegnebrett), model (lagringspunkt), og figur (standard valg)
	TegnePanel tegnepanel = new TegnePanel();
	Model model = new Model();
	Oval figur = new Oval(); // til  sette standard valg

	// hoved vindu strrelse
	int w = 800, h=400;

	// meny sjekkbokser
	JRadioButtonMenuItem ji_fao;
	JRadioButtonMenuItem ji_far;
	JRadioButtonMenuItem ji_fal;
	JRadioButtonMenuItem ji_tf;
	JRadioButtonMenuItem ji_tk;
	JCheckBoxMenuItem ji_di;

	// definer strrelse vindu tekstfelter
	JTextField tf_dx;
	JTextField tf_dy;
	JFrame f_storrelse;

	// dragging instansvariabler
	boolean firstdragevt = false;
	boolean dragging = false;

	// liste hyre instans
	JList liste;

	// venstre verkty
	JCheckBox oval;
	JCheckBox rektangel;
	JCheckBox linje;

	JCheckBox velg;
	JCheckBox tilfeldig;

	JCheckBox kontur;
	JCheckBox fylt;

	JCheckBox fargepalette;

	// hent bilder
	ClassLoader cldr;

	View () {
		// sett opp model som standard tegne arkiv
		tegnepanel.setTegnepanel(model);

		// endre default look and feel
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception unused) {}

		// legg til listeners til tegnepanel
		tegnepanel.addMouseListener(new mouseListener());
		tegnepanel.addMouseMotionListener(new mousemotionListener());

		// hent bilder fra classloader
		cldr = this.getClass().getClassLoader();

		// lage gui - ikke kommentert
		panelkomponenter();

		// lag menybar
		meny();

		// standard verdier fra grunnfigur, settes opp i model objektet
		model.setFarge(figur.getFarge());
		fargepalette.setBackground(figur.getFarge());
		model.setDx(figur.getDx());
		model.setDy(figur.getDy());
		model.setTegnemate(figur.isTegnemate());
		kontur.setBackground(model.getSelectedColor());

		model.setFasong(Fasong.REKTANGEL);
		rektangel.setBackground(model.getSelectedColor());

		ji_far.setSelected(true);
		velg.setBackground(model.getSelectedColor());

		model.setDirectinput(true);
		ji_di.setSelected(true);

		ji_tk.setSelected(true);

		// standard frame innstillinger
		this.setSize(w,h);
		this.setTitle("Tegnebrett");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setLocation(300,200);
		this.setVisible(true);
	}

	private void panelkomponenter() {
		Container content = this.getContentPane();
		content.setLayout(new BorderLayout());

		JPanel cp = new JPanel();
		cp.setLayout(new BorderLayout(5,5));
		cp.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

		JPanel left = new JPanel();
		left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));

		Border border = BorderFactory.createEtchedBorder();
		Border empty = BorderFactory.createEmptyBorder(2,2,2,2);
		Border com = BorderFactory.createCompoundBorder(border,empty);

		JPanel fasong = new JPanel();
		fasong.setMaximumSize(new Dimension(35,80));
		fasong.setLayout(new GridLayout(0,1,2,2));
		fasong.setBorder(com);

		oval = new JCheckBox(new ImageIcon(cldr.getResource("images/oval.png")));
		oval.setToolTipText("Oval figur (O)");
		oval.addActionListener(new fasongovalListener());
		rektangel = new JCheckBox(new ImageIcon(cldr.getResource("images/rektangel.png")));
		rektangel.setToolTipText("Rektangulr figur (R)");
		rektangel.addActionListener(new fasongrektangelListener());
		linje = new JCheckBox(new ImageIcon(cldr.getResource("images/linje.png")));
		linje.setToolTipText("Linje verkty (L)");
		linje.addActionListener(new fasonglinjeListener());
		fasong.add(oval);
		fasong.add(rektangel);
		fasong.add(linje);

		JPanel farge = new JPanel();
		farge.setMaximumSize(new Dimension(35,55));
		farge.setLayout(new GridLayout(0,1,2,2));
		farge.setBorder(com);

		velg = new JCheckBox(new ImageIcon(cldr.getResource("images/farge.png")));
		velg.setToolTipText("Fargevalg (V)");
		velg.addActionListener(new fargeListener());
		tilfeldig = new JCheckBox(new ImageIcon(cldr.getResource("images/tilfeldig.png")));
		tilfeldig.setToolTipText("Tilfeldig farge (T)");
		tilfeldig.addActionListener(new tilfeldigfargeListener());
		farge.add(velg);
		farge.add(tilfeldig);

		JPanel tegnemate = new JPanel();
		tegnemate.setMaximumSize(new Dimension(35,55));
		tegnemate.setLayout(new GridLayout(0,1,2,2));
		tegnemate.setBorder(com);

		kontur = new JCheckBox(new ImageIcon(cldr.getResource("images/kontur.png")));
		kontur.setToolTipText("Kontur figur, hul innsidig (K)");
		kontur.addActionListener(new tegnematekonturListener());
		fylt = new JCheckBox(new ImageIcon(cldr.getResource("images/fylt.png")));
		fylt.setToolTipText("Fylt figur (F)");
		fylt.addActionListener(new tegnematefyltListener());
		tegnemate.add(kontur);
		tegnemate.add(fylt);

		JPanel fargevalg = new JPanel();
		fargevalg.setMaximumSize(new Dimension(35,30));
		fargevalg.setLayout(new GridLayout(0,1,2,2));
		fargevalg.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		fargepalette = new JCheckBox(new ImageIcon(""));
		fargepalette.addActionListener(new fargeListener());
		fargevalg.add(fargepalette);

		left.add(Box.createRigidArea(new Dimension(4, 4)));
		left.add(fasong);
		left.add(Box.createRigidArea(new Dimension(4, 8)));
		left.add(farge);
		left.add(Box.createRigidArea(new Dimension(4, 8)));
		left.add(tegnemate);
		left.add(Box.createRigidArea(new Dimension(4, 8)));
		left.add(fargevalg);

		cp.add(left,BorderLayout.WEST);

		cp.add(new JLabel("Tegnebrett:"),BorderLayout.NORTH);

		JPanel right = new JPanel();
		right.setLayout(new BorderLayout(5,5));
		liste=new JList(model.getListeModel());
		liste.setFixedCellWidth(160);
		JScrollPane jscrolliste = new JScrollPane(liste);
		jscrolliste.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		right.add(jscrolliste,BorderLayout.CENTER);

		JButton slett = new JButton("Slett");
		slett.addActionListener(new slettfraListeListener());
		right.add(slett,BorderLayout.SOUTH);

		cp.add(tegnepanel,BorderLayout.CENTER);
		cp.add(right,BorderLayout.EAST);
		content.add(cp,BorderLayout.CENTER);
	}

	private void meny() {
		// meny
		JMenuBar jm = new JMenuBar();
		this.setJMenuBar(jm);

		// fil
		JMenu ji_fil = new JMenu("Fil");
		ji_fil.setMnemonic('F');
		jm.add(ji_fil);
		JMenuItem ji_ny = new JMenuItem("Nytt tegnebrett");		
		ji_ny.addActionListener(new nyttbildeListener());
		ji_ny.setMnemonic('N');
		ji_ny.setAccelerator(KeyStroke.getKeyStroke("control N"));
		ji_fil.add(ji_ny);
		ji_fil.addSeparator();
		JMenuItem ji_lb = new JMenuItem("Lagre bilde...");		
		ji_lb.addActionListener(new lagreListener());
		ji_lb.setMnemonic('L');
		ji_lb.setAccelerator(KeyStroke.getKeyStroke("control S"));
		ji_fil.add(ji_lb);

		JMenu ji_m = new JMenu("Modell");		
		ji_m.setMnemonic('M');
		ji_fil.add(ji_m);
		JMenuItem ji_ml = new JMenuItem("Lagre...");		
		ji_ml.setMnemonic('L');
		ji_ml.setAccelerator(KeyStroke.getKeyStroke("control shift S"));
		ji_ml.addActionListener(new modelllagreListener());
		ji_m.add(ji_ml);
		JMenuItem ji_ma = new JMenuItem("pne...");		
		ji_ma.setMnemonic('');
		ji_ma.setAccelerator(KeyStroke.getKeyStroke("control O"));
		ji_ma.addActionListener(new modellhentListener());
		ji_m.add(ji_ma);
		ji_fil.addSeparator();

		JMenuItem ji_a = new JMenuItem("Avslutt");		
		ji_a.addActionListener(new avsluttListener());
		ji_a.setMnemonic('A');
		ji_a.setAccelerator(KeyStroke.getKeyStroke("control Q"));

		ji_fil.add(ji_a);

		// format
		JMenu ji_f = new JMenu("Format");
		ji_f.setMnemonic('o');
		jm.add(ji_f);

		JMenu ji_fa  = new JMenu("Fasong");
		ji_fa.setMnemonic('F');
		ji_f.add(ji_fa);
		ji_fao = new JRadioButtonMenuItem("Oval");
		ji_fao.setMnemonic('O');
		ji_fao.setAccelerator(KeyStroke.getKeyStroke("O"));
		ji_fao.addActionListener(new fasongovalListener());
		ji_fa.add(ji_fao);
		ji_far = new JRadioButtonMenuItem("Rektangel");
		ji_far.setMnemonic('R');
		ji_far.setAccelerator(KeyStroke.getKeyStroke("R"));
		ji_far.addActionListener(new fasongrektangelListener());
		ji_fa.add(ji_far);
		ji_fal = new JRadioButtonMenuItem("Linje");		
		ji_fal.setMnemonic('L');
		ji_fal.setAccelerator(KeyStroke.getKeyStroke("L"));
		ji_fal.addActionListener(new fasonglinjeListener());
		ji_fa.add(ji_fal);

		JMenu ji_ff  = new JMenu("Farge");
		ji_ff.setMnemonic('e');
		ji_f.add(ji_ff);
		JMenuItem ji_fvf = new JMenuItem("Velg farge...");		
		ji_fvf.setMnemonic('V');
		ji_fvf.setAccelerator(KeyStroke.getKeyStroke("V"));
		ji_fvf.addActionListener(new fargeListener());
		ji_ff.add(ji_fvf);
		JMenuItem ji_ftf = new JMenuItem("Tilfeldig farge");		
		ji_ftf.setMnemonic('T');
		ji_ftf.setAccelerator(KeyStroke.getKeyStroke("T"));
		ji_ftf.addActionListener(new tilfeldigfargeListener());
		ji_ff.add(ji_ftf);

		JMenu ji_t  = new JMenu("Tegnemte");
		ji_t.setMnemonic('T');
		ji_f.add(ji_t);
		ji_tk = new JRadioButtonMenuItem("Kontur");		
		ji_tk.setMnemonic('K');
		ji_tk.setAccelerator(KeyStroke.getKeyStroke("K"));
		ji_tk.addActionListener(new tegnematekonturListener());
		ji_t.add(ji_tk);
		ji_tf = new JRadioButtonMenuItem("Fylt");		
		ji_tf.setMnemonic('F');
		ji_tf.setAccelerator(KeyStroke.getKeyStroke("F"));
		ji_tf.addActionListener(new tegnematefyltListener());
		ji_t.add(ji_tf);

		// strrelse
		JMenu ji_s = new JMenu("Dimensjon");
		ji_s.setMnemonic('D');
		jm.add(ji_s);
		JMenuItem ji_dx = new JMenuItem("Definer strrelse...");	
		ji_dx.setMnemonic('s');
		ji_dx.setAccelerator(KeyStroke.getKeyStroke("D"));
		ji_dx.addActionListener(new setupdxListener());
		ji_s.add(ji_dx);
		/*
		JMenu ji_setup_dimensjon  = new JMenu("Definer strrelse");
		ji_setup_dimensjon.setMnemonic('e');
		ji_setup.add(ji_setup_dimensjon);
		JMenuItem ji_dimension_dx = new JMenuItem("Bredde (dx)...");	
		ji_dimension_dx.setMnemonic('x');
		ji_dimension_dx.setAccelerator(KeyStroke.getKeyStroke("X"));
		ji_dimension_dx.addActionListener(new setupdxListener());
		ji_setup_dimensjon.add(ji_dimension_dx);
		JMenuItem ji_dimension_dy = new JMenuItem("Hyde (dy)...");		
		ji_dimension_dy.setMnemonic('y');
		ji_dimension_dy.setAccelerator(KeyStroke.getKeyStroke("Y"));
		ji_dimension_dy.addActionListener(new setupdyListener());
		ji_setup_dimensjon.add(ji_dimension_dy);
		 */
		ji_di = new JCheckBoxMenuItem("Bruk musen");		
		ji_di.setMnemonic('m');
		ji_di.setAccelerator(KeyStroke.getKeyStroke("M"));
		ji_di.addActionListener(new directinputListener());
		ji_s.add(ji_di);
	}


	/**
	 * Fil filter. Filtere kun ut .tfm filer.
	 * @author Administrator
	 *
	 */
	private class modellFilter extends FileFilter {
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".tfm");
		}
		public String getDescription() {
			return ".tfm - TegneFigur Modeller";
		}
	}

	/**
	 * Lagre som bilde, jpg.
	 * @author Administrator
	 *
	 */
	private class lagrebildejpgFilter extends FileFilter {
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".jpg");
		}
		public String getDescription() {
			return ".jpg - JPEG	 Bilde";
		}
	}

	/**
	 * Lagre som bilde, gif
	 * @author Administrator
	 *
	 */
	private class lagrebildegifFilter extends FileFilter {
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".gif");
		}
		public String getDescription() {
			return ".gif - GIF Bilde";
		}
	}

	/**
	 * Lagre som bilde, png.
	 * @author Administrator
	 *
	 */
	private class lagrebildepngFilter extends FileFilter {
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".png");
		}
		public String getDescription() {
			return ".png - PNG Bilde";
		}
	}

	/**
	 * Lag en tilfeldig farge.
	 * @return farge som ble laget.
	 */
	private static Color randomColor() {
		return new Color((int) (Math.random() * 256),  // Red
				(int) (Math.random() * 256),  // Green
				(int) (Math.random() * 256)); // Blue
	}
	/**
	 * Sett start verdier for Dx og Dy ved definering av strrelse.
	 */
	private void settStartVerdi() {
		if (model.isDirectinput()) {
			model.setDx(figur.getDx());
			model.setDy(figur.getDy());
			model.setDirectinput(false);				
		}
	}

	/**
	 * Lag en ny arrayliste og overskriv den nvrende.
	 */
	private void nyttbilde() {
		ArrayList<TegneFigur> al=new ArrayList<TegneFigur>();
		model.setTegninger(al);
		tegnepanel.repaint();
	}

	/**
	 * Lagre tegningen som et bilde. Lagre enten som jpg, gif, eller png. Fil velger vises.
	 */
	private void lagreBilde() {

		JFileChooser jf=new JFileChooser();
		FileFilter filter_jpg = new lagrebildejpgFilter();
		FileFilter filter_gif = new lagrebildegifFilter();
		FileFilter filter_png = new lagrebildepngFilter();

		jf.addChoosableFileFilter(filter_jpg);
		jf.addChoosableFileFilter(filter_gif);
		jf.addChoosableFileFilter(filter_png);
		jf.setFileSelectionMode(JFileChooser.FILES_ONLY); // FILES_AND_DIRECTORIES
		int returnVal = jf.showSaveDialog(null);

		if (returnVal==JFileChooser.APPROVE_OPTION){
			// finn extension
			String valgtfilter = jf.getFileFilter().getDescription();
			int finn = valgtfilter.indexOf(" ");
			String ext = valgtfilter.substring(1,finn);
			if (ext.equals("")) ext = "png";
			// fil adresse
			String filepath = jf.getSelectedFile().getPath();
			String newfile;
			if (filepath.lastIndexOf(".jpg")!=-1||filepath.lastIndexOf(".gif")!=-1||filepath.lastIndexOf(".png")!=-1) {
				newfile = filepath;
			} else {
				newfile = filepath+"."+ext;
			}
			File fs = new File(newfile);
			jf.setSelectedFile(fs);
			File f = jf.getSelectedFile();			
			try {
				if (f.exists()) f.delete();
				BufferedImage bi = tegnepanel.getBuffimage(); // retrieve image
				ImageIO.write(bi, ext, f);
			} catch (IOException e) {
			}

		}
	}

	/**
	 * Lagre objekt som fil. Inneholder alle instanser av tegningene.
	 */
	private void lagreModell() {

		JFileChooser jf=new JFileChooser();
		FileFilter filter = new modellFilter();

		jf.addChoosableFileFilter(filter);
		jf.setFileSelectionMode(JFileChooser.FILES_ONLY); // FILES_AND_DIRECTORIES
		int returnVal = jf.showSaveDialog(null);
		if (returnVal==JFileChooser.APPROVE_OPTION){
			// finn file extension
			String valgtfilter = jf.getFileFilter().getDescription();
			int finn = valgtfilter.indexOf(" ");
			String ext = valgtfilter.substring(0,finn);
			// fil adresse
			String filepath = jf.getSelectedFile().getPath();
			String newfile;
			if (filepath.lastIndexOf(".tfm")!=-1) {
				newfile = filepath;
			} else {
				newfile = filepath+ext;
			}
			File fs = new File(newfile);
			jf.setSelectedFile(fs);
			File f = jf.getSelectedFile();
			try{
				if (f.exists()) f.delete();
				ObjectOutputStream out =
					new ObjectOutputStream(
							new FileOutputStream(f));
				out.writeObject(model.getTegninger());//h er serializable

				out.close();
				System.out.println("Lagret modell til: "+f.toString());
			} catch(FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Hent opp objekt fra fil. Inneholder alle instansene av tegningene.
	 */
	private void henteModell() {

		JFileChooser jf=new JFileChooser();
		jf.setFileFilter(new modellFilter());
		jf.setFileSelectionMode(JFileChooser.FILES_ONLY); // FILES_AND_DIRECTORIES
		int returnVal = jf.showOpenDialog(null);
		if (returnVal==JFileChooser.APPROVE_OPTION){
			File f = jf.getSelectedFile();

			Object m;
			try{
				FileInputStream fileIn = new FileInputStream(f);
				ObjectInputStream in = new ObjectInputStream(fileIn);
				m = in.readObject();
				in.close();
				fileIn.close();
				System.out.println("Hentet modell fra: "+f.toString());

				ArrayList<TegneFigur> ny = (ArrayList<TegneFigur>) m;

				/*	//model.al.clear();
				for (TegneFigur tf:ny) {
					//model.add(tf);
					System.out.println("getclass: "+tf.getClass().getName());
					try {
						Rektangel r = (Rektangel)tf;
						System.out.println("Rektangel: "+r.getFasong()+" x: "+r.getX());
					} catch (Exception er) {
						try {
							Linje l = (Linje)tf;
							System.out.println("Linje: "+l.getFasong()+" x: "+l.getX());
						} catch (Exception el) {
							try {
								Oval o = (Oval)tf;
								System.out.println("Oval: "+o.getFasong()+" x: "+o.getX());
							} catch (Exception eo) {

							}
						}
					}
					System.out.println("TegneFigur: "+tf.toString());
				} */
				model.slettAlle();
				model.setTegninger(ny);
				tegnepanel.repaint();
			} catch (Exception e) {
				System.out.println("Feil: "+e.getMessage());
			}
		}
	}

	/**
	 * Velg farge fra fargehjul. Endringen skjer med det samme ved klikk.
	 */
	private void velgFarge() {

		//Color farge = Color.GREEN;
		//model.setFarge(farge);						

		int x = this.getX();
		int y = this.getY();

		JFrame f = new JFrame();
		f.setLocation(x+w+5,y);
		f.setTitle("Velg farge");
		f.setResizable(false);

		Color curColor = model.getFarge();
		JColorChooser Color = new JColorChooser(curColor);
		Container cp=f.getContentPane();
		cp.add(Color,BorderLayout.CENTER);

		ColorSelectionModel cmodel = Color.getSelectionModel();
		cmodel.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt){
				ColorSelectionModel cmodel = (ColorSelectionModel)evt.getSource();
				model.setFarge(cmodel.getSelectedColor());
				fargepalette.setBackground(model.getFarge());
			}
		});
		f.setSize(500,300);
		f.setVisible(true);
	}

	/**
	 * Definer strrelse p tegnefigur. Velg heltall verdi for Dx og Dy.
	 */
	private void definerStorrelse() {
		int x = this.getX();
		int y = this.getY();

		f_storrelse = new JFrame();
		f_storrelse.setLocation(x+w/2,y+h/2);
		f_storrelse.setTitle("Strrelse");
		f_storrelse.setResizable(false);
		f_storrelse.setSize(225,115);


		Container cp=f_storrelse.getContentPane();
		JPanel jp = new JPanel();
		jp.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		jp.setLayout(new GridLayout(3,2,3,3));

		jp.add(new JLabel("Bredde (Dx):"));
		tf_dx = new JTextField();
		tf_dx.setText(""+model.getFinalDx());
		jp.add(tf_dx);
		jp.add(new JLabel("Hyde (Dy):"));
		tf_dy = new JTextField();
		tf_dy.setText(""+model.getFinalDy());
		jp.add(tf_dy);
		jp.add(new JLabel(""));
		JButton jb = new JButton("Ok");
		f_storrelse.getRootPane().setDefaultButton(jb);
		jp.add(jb);

		cp.add(jp,BorderLayout.CENTER);

		jb.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt){

				Pattern pat = Pattern.compile("\\d");
				Matcher m_dx = pat.matcher(tf_dx.getText());
				Matcher m_dy = pat.matcher(tf_dy.getText());
				if (m_dx.find() && m_dy.find()) {
					try {
						model.setDx(Integer.parseInt(tf_dx.getText()));
						model.setDy(Integer.parseInt(tf_dy.getText()));
						model.setFinalDx(Integer.parseInt(tf_dx.getText()));
						model.setFinalDy(Integer.parseInt(tf_dy.getText()));

					} catch (Exception e) { }
				}
				f_storrelse.setVisible(false);

			}
		});
		f_storrelse.setVisible(true);
	}

	/**
	 * Tegn figur ved dragging. Disse figurene lagres ikke under dragging, kun ved mus knapp slipp.
	 */
	private void tegnfigurVedDragging() {

		if (model.isDirectinput()) {
			if (firstdragevt) {
				firstdragevt = false;
				model.slettIndexNr(1);
			}

			Fasong fasong = model.getFasong();
			switch (fasong) {
			case LINJE:
				Linje l = new Linje();
				l.setFarge(model.getFarge());
				// figur gr tilbake mot 0				
				if (model.getDx() < model.getX()) {
					l.setDx(model.getX()-model.getDx()); // x2 koord (finner lengde i Linje objekt)
					l.setX(model.getX()); // x1 koord
					l.setDy(model.getY()-model.getDy()); // y2 koord (finner lengde i Linje objekt)
					l.setY(model.getY()); // y1	koord				
				// figur gr fra 0 utover
				} else {
					l.setDx(model.getDx()-model.getX());
					l.setX(model.getDx());
					l.setDy(model.getDy()-model.getY());
					l.setY(model.getDy());
				}
				// for definer strrelse tekstfelt verdier
				model.setFinalDx(l.getDx());
				model.setFinalDy(l.getDy());

				model.add(l,dragging);
				break;
			case OVAL:
				Oval  o = new Oval();			
				o.setTegnemate(model.isTegnemate());
				o.setFarge(model.getFarge());
				// figur gr tilbake mot 0
				if (model.getDx() < model.getX()) {
					o.setDx(model.getX()-model.getDx()); // lenge mellom x og dx
					o.setX(model.getX()); // start punkt
					// figur gr fra 0 utover
				} else {
					o.setDx(model.getDx()-model.getX()); // lengde mellom dx og x
					o.setX(model.getDx()); // start punkt
				}
				if (model.getDy() < model.getY()) {
					o.setDy(model.getY()-model.getDy());
					o.setY(model.getY());					
				} else {
					o.setDy(model.getDy()-model.getY());
					o.setY(model.getDy());
				}
				// for definer strrelse tekstfelt verdier
				model.setFinalDx(o.getDx());
				model.setFinalDy(o.getDy());

				model.add(o,dragging);
				break;
			case REKTANGEL:
				Rektangel r = new Rektangel();
				r.setTegnemate(model.isTegnemate());
				r.setFarge(model.getFarge());
				if (model.getDx() < model.getX()) {
					r.setDx(model.getX()-model.getDx());
					r.setX(model.getX());
				} else {
					r.setDx(model.getDx()-model.getX());
					r.setX(model.getDx());
				}
				if (model.getDy() < model.getY()) {
					r.setDy(model.getY()-model.getDy());
					r.setY(model.getY());					
				} else {
					r.setDy(model.getDy()-model.getY());
					r.setY(model.getDy());
				}
				// for definer strrelse tekstfelt verdier - ferdig regnet ut fra ovenfor
				model.setFinalDx(r.getDx());
				model.setFinalDy(r.getDy());

				model.add(r,dragging);
				break;
			}


			tegnepanel.repaint();
		}
	}

	/**
	 * Tegn figur med definert strrelse. Brukes ogs for  tegne siste bilde ved dragging.
	 * @param ev Mus event, inneholder posisjon til peker.
	 */
	private void tegnfigurMedDimensjoner(MouseEvent ev) {
		// hvis musen er brukt, sett start verdiene ved frste muse nedtrykk
		if (model.isDirectinput()) {
			model.setX(ev.getX());
			model.setY(ev.getY());
		} else {
			Fasong fasong = model.getFasong();
			switch (fasong) {
			case LINJE:
				Linje l = new Linje();
				l.setFarge(model.getFarge());
				/*
			l.setDx(model.getDx());
			l.setDy(model.getDy());
			l.setX(ev.getX());
			l.setY(ev.getY());
				 */

				if ((model.getFinalDx()+ev.getX()) < ev.getX()) {
					l.setDx(ev.getX()-(model.getFinalDx()+ev.getX()));
					l.setX(ev.getX());
					l.setDy(ev.getY()-(model.getFinalDy()+ev.getY()));
					l.setY(ev.getY());					
				} else {
					l.setDx((model.getFinalDx()+ev.getX())-ev.getX());
					l.setX((model.getFinalDx()+ev.getX()));
					l.setDy((model.getFinalDy()+ev.getY())-ev.getY());
					l.setY((model.getFinalDy()+ev.getY()));
				}

				model.add(l,firstdragevt);
				break;
			case OVAL:
				Oval  o = new Oval();			
				o.setTegnemate(model.isTegnemate());
				o.setFarge(model.getFarge());
				o.setDx(model.getFinalDx());
				o.setDy(model.getFinalDy());
				o.setX(ev.getX());
				o.setY(ev.getY());

				model.add(o,firstdragevt);
				break;
			case REKTANGEL:
				Rektangel r = new Rektangel();
				r.setTegnemate(model.isTegnemate());
				r.setFarge(model.getFarge());
				r.setDx(model.getFinalDx());
				r.setDy(model.getFinalDy());
				r.setX(ev.getX());
				r.setY(ev.getY());

				model.add(r,firstdragevt);
				break;
			}
			tegnepanel.repaint();
		}
	}

	/**
	 * Tegner animasjon ved dragging av musen for  lag ny figur. Under dragging slettes de siste tegningene.
	 * @param ev
	 */
	private void tegnfigurOppdaterDragPos(MouseEvent ev) {
		if (model.isDirectinput()) {
			// sett oppdatert drag posisjon
			// brukes til  regne endelig posisjon senere
			model.setDx(ev.getX());
			model.setDy(ev.getY());

			// tegn live dragging animasjon
			tegnfigurVedDragging();
			// hvis kjrt en gang tidligere (betyr 2  dragging animasjoner inkl. denne finnes), slekk den frste
			if (firstdragevt) {
				model.slettIndexNr(2);
				// ved frste kjring, sett sann
			} else {
				firstdragevt = true;
			}
		}
	}

	/**
	 * Sletter tegninger som er valgt fra jlisten til hyre i hovedvinduet.
	 */
	private void slettTegninger() {
		Object tegninger[] = liste.getSelectedValues();

		for (Object t:tegninger) {
			if (t!=null) {
				TegneFigur tf = (TegneFigur)t;
				model.slettTegning(tf);
				tegnepanel.repaint();
			}

		}
	}

	// LISTENERS
	private class nyttbildeListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// nytt bilde
			nyttbilde();
		}
	}
	private class lagreListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// lagre
			lagreBilde();
		}
	}
	private class modelllagreListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// modell lagre
			lagreModell();
		}
	}
	private class modellhentListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// modell pne
			henteModell();
		}
	}

	private class avsluttListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// modell pne
			System.exit(0);
		}
	}
	private class fasongovalListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// fasong oval
			rektangel.setBackground(null);
			linje.setBackground(null);
			oval.setBackground(model.getSelectedColor());			
			oval.setSelected(true);			
			ji_fao.setSelected(true);
			ji_far.setSelected(false);
			ji_fal.setSelected(false);
			model.setFasong(Fasong.OVAL);									
		}
	}
	private class fasongrektangelListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// fasong rektangel
			oval.setBackground(null);
			linje.setBackground(null);
			rektangel.setBackground(model.getSelectedColor());
			rektangel.setSelected(true);
			ji_far.setSelected(true);
			ji_fao.setSelected(false);
			ji_fal.setSelected(false);
			model.setFasong(Fasong.REKTANGEL);						
		}
	}
	private class fasonglinjeListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// fasong linje
			oval.setBackground(null);
			rektangel.setBackground(null);
			linje.setBackground(model.getSelectedColor());
			linje.setSelected(true);
			ji_fal.setSelected(true);
			ji_fao.setSelected(false);
			ji_far.setSelected(false);
			model.setFasong(Fasong.LINJE);									

			// endre tegnemate til kontur for linje, ikke mulig  fylle
			fylt.setBackground(null);
			kontur.setBackground(model.getSelectedColor());			
			kontur.setSelected(true);
			ji_tf.setSelected(false);
			ji_tk.setSelected(true);
			model.setTegnemate(true);
		}
	}
	private class fargeListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// farge
			tilfeldig.setBackground(null);
			velg.setBackground(model.getSelectedColor());			
			velg.setSelected(true);
			velgFarge();
		}
	}

	private class tilfeldigfargeListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// farge random
			velg.setBackground(null);
			tilfeldig.setBackground(model.getSelectedColor());			
			tilfeldig.setSelected(true);
			model.setFarge(randomColor());
			fargepalette.setBackground(model.getFarge());
		}
	}
	private class tegnematekonturListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// tegnemte kontur
			fylt.setBackground(null);
			kontur.setBackground(model.getSelectedColor());			
			kontur.setSelected(true);
			ji_tk.setSelected(true);
			ji_tf.setSelected(false);
			model.setTegnemate(true);
		}
	}
	private class tegnematefyltListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// tegnemte fylt
			if (model.getFasong()!=Fasong.LINJE) {
				kontur.setBackground(null);
				fylt.setBackground(model.getSelectedColor());			
				fylt.setSelected(true);
				ji_tf.setSelected(true);
				ji_tk.setSelected(false);
				model.setTegnemate(false);			
			} else {
				// sett tilbake til ikke valg i menyen
				ji_tf.setSelected(false);
			}
		}
	}
	private class setupdxListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// setup dx
			// sjekk om musen er brukt tidligere
			ji_di.setSelected(false);
			model.setDirectinput(false);

			//settStartVerdi();
			definerStorrelse();
			/*
			defaultdxdy();
			String dx = JOptionPane.showInputDialog(null, "Oppgi dx verdi (heltall):");

			Pattern pat = Pattern.compile("\\d");
			Matcher m = pat.matcher(dx);
			if (m.find()) {
				try {
					model.setDx(Integer.parseInt(dx));
				} catch (Exception e) { }
			}
			 */
		}
	}
	/*
	private class setupdyListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// setup dy
			// sjekk om musen er brukt tidligere
			defaultdxdy();
			String dy = JOptionPane.showInputDialog(null, "Oppgi dy verdi (heltall):");
			Pattern pat = Pattern.compile("\\d");
			Matcher m = pat.matcher(dy);
			if (m.find()) {
				try {
					model.setDy(Integer.parseInt(dy));
				} catch (Exception e) { }
			}
		}
	} */
	private class directinputListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// strrelse direkte input fra mus
			if (!ji_di.isSelected()) {
				//settStartVerdi();
				ji_di.setSelected(false);
				model.setDirectinput(false);
			} else {
				ji_di.setSelected(true);
				model.setDirectinput(true);
			}
		}
	}

	// Mouse listener
	private class mouseListener implements MouseListener {
		public void mouseClicked(MouseEvent ev) { System.out.println("Ny mus handling: clicked! Clicks: "+ev.getClickCount()); }
		public void mouseEntered(MouseEvent ev){ 
			System.out.println("Ny mus handling: entered!"); 
		}
		public void mouseExited(MouseEvent ev){
			System.out.println("Ny mus handling: exited!"); 
		}
		public void mouseReleased(MouseEvent ev){
			dragging = false;
			tegnfigurVedDragging();
			System.out.println("Ny mus handling: released!");
		}
		public void mousePressed(MouseEvent ev){
			/* // Legg til tom tegning for dragging
			Rektangel g = new Rektangel();
			g.setDx(0);g.setDy(0);g.setX(0);g.setY(0);
			model.add(g); */
			tegnfigurMedDimensjoner(ev);
			System.out.println("Ny mus handling: pressed!");	
		}
		public void mouseMoved(MouseEvent ev) {	System.out.println(ev.getX()+"x"+ev.getY()); }
	}

	private class mousemotionListener implements MouseMotionListener {
		public void mouseMoved(MouseEvent ev) {	
			//System.out.println("Moved: "+ev.getX()+"x"+ev.getY()); 
		}
		public void mouseDragged(MouseEvent ev) {
			dragging = true;
			tegnfigurOppdaterDragPos(ev);
			System.out.println("Dragged: "+ev.getX()+"x"+ev.getY());
		}
	}

	private class slettfraListeListener implements ActionListener {
		public void actionPerformed(ActionEvent ev) {
			// slett fra tegning
			slettTegninger();
		}
	}
}
